/**
*     Copyright (c) 2023, Nations Technologies Inc.
* 
*     All rights reserved.
*
*     This software is the exclusive property of Nations Technologies Inc. (Hereinafter 
* referred to as NATIONS). This software, and the product of NATIONS described herein 
* (Hereinafter referred to as the Product) are owned by NATIONS under the laws and treaties
* of the People's Republic of China and other applicable jurisdictions worldwide.
*
*     NATIONS does not grant any license under its patents, copyrights, trademarks, or other 
* intellectual property rights. Names and brands of third party may be mentioned or referred 
* thereto (if any) for identification purposes only.
*
*     NATIONS reserves the right to make changes, corrections, enhancements, modifications, and 
* improvements to this software at any time without notice. Please contact NATIONS and obtain 
* the latest version of this software before placing orders.

*     Although NATIONS has attempted to provide accurate and reliable information, NATIONS assumes 
* no responsibility for the accuracy and reliability of this software.
* 
*     It is the responsibility of the user of this software to properly design, program, and test 
* the functionality and safety of any application made of this information and any resulting product. 
* In no event shall NATIONS be liable for any direct, indirect, incidental, special,exemplary, or 
* consequential damages arising in any way out of the use of this software or the Product.
*
*     NATIONS Products are neither intended nor warranted for usage in systems or equipment, any
* malfunction or failure of which may cause loss of human life, bodily injury or severe property 
* damage. Such applications are deemed, "Insecure Usage".
*
*     All Insecure Usage shall be made at user's risk. User shall indemnify NATIONS and hold NATIONS 
* harmless from and against all claims, costs, damages, and other liabilities, arising from or related 
* to any customer's Insecure Usage.

*     Any express or implied warranty with regard to this software or the Product, including,but not 
* limited to, the warranties of merchantability, fitness for a particular purpose and non-infringement
* are disclaimed to the fullest extent permitted by law.

*     Unless otherwise explicitly permitted by NATIONS, anyone may not duplicate, modify, transcribe
* or otherwise distribute this software for any purposes, in whole or in part.
*
*     NATIONS products and technologies shall not be used for or incorporated into any products or systems
* whose manufacture, use, or sale is prohibited under any applicable domestic or foreign laws or regulations. 
* User shall comply with any applicable export control laws and regulations promulgated and administered by 
* the governments of any countries asserting jurisdiction over the parties or transactions.
**/

/**
*\*\file main.h
*\*\author Nations
*\*\version v1.0.0
*\*\copyright Copyright (c) 2022, Nations Technologies Inc. All rights reserved.
**/
#ifndef __MAIN_H__
#define __MAIN_H__

#ifdef __cplusplus
extern "C" {
#endif

#include "n32g05x.h"
#include "n32g05x_rcc.h"
#include "n32g05x_gpio.h"
#include "n32g05x_uart.h"
#include "n32g05x_dma.h"

#define _UART1_UART2_

#ifdef _UART1_UART2_
#define UARTy            UART1
#define UARTy_CLK        RCC_APB2_PERIPH_UART1
#define UARTy_GPIO       GPIOA
#define UARTy_GPIO_CLK   RCC_APB2_PERIPH_GPIOA
#define UARTy_RxPin      GPIO_PIN_3
#define UARTy_TxPin      GPIO_PIN_2
#define UARTy_Rx_GPIO_AF GPIO_AF5_UART1
#define UARTy_Tx_GPIO_AF GPIO_AF5_UART1
#define UARTy_APBxClkCmd RCC_EnableAPB2PeriphClk
#define UARTy_DAT_Base         (UART1_BASE + 0x10)
#define UARTy_Tx_DMA_Channel   DMA_CH4
#define UARTy_Tx_DMA_FLAG      DMA_FLAG_TC4
#define UARTy_Rx_DMA_Channel   DMA_CH5
#define UARTy_Rx_DMA_FLAG      DMA_FLAG_TC5
#define UARTy_Tx_DMA_REMAP     DMA_REMAP_UART1_TX
#define UARTy_Rx_DMA_REMAP     DMA_REMAP_UART1_RX

#define UARTz            UART2
#define UARTz_GPIO       GPIOA
#define UARTz_CLK        RCC_APB2_PERIPH_UART2
#define UARTz_GPIO_CLK   RCC_APB2_PERIPH_GPIOA
#define UARTz_RxPin      GPIO_PIN_7
#define UARTz_TxPin      GPIO_PIN_6
#define UARTz_Rx_GPIO_AF GPIO_AF4_UART2
#define UARTz_Tx_GPIO_AF GPIO_AF4_UART2
#define UARTz_APBxClkCmd RCC_EnableAPB2PeriphClk
#define UARTz_DAT_Base         (UART2_BASE + 0x10)
#define UARTz_Tx_DMA_Channel   DMA_CH1
#define UARTz_Tx_DMA_FLAG      DMA_FLAG_TC1
#define UARTz_Rx_DMA_Channel   DMA_CH2
#define UARTz_Rx_DMA_FLAG      DMA_FLAG_TC2
#define UARTz_Tx_DMA_REMAP     DMA_REMAP_UART2_TX
#define UARTz_Rx_DMA_REMAP     DMA_REMAP_UART2_RX
#endif

typedef enum
{
    FAILED = 0,
    PASSED = !FAILED
} TestStatus;

void RCC_Configuration(void);
void GPIO_Configuration(void);
void DMA_Configuration(void);
TestStatus Buffercmp(uint8_t* pBuffer1, uint8_t* pBuffer2, uint16_t BufferLength);

#ifdef __cplusplus
}
#endif

#endif /* __MAIN_H__ */
